``````````````````````````````````
/*
  DRAFT WORK IN PROGRESS 
  PV relay using power from mains (imp/exp) to switch a relay.  
    Adaptation of emonTxV3.4 Discrete Sampling
    This version is used to power a circulation pump that de-startifies (mixes up) a hot water cylinder. 
    The cylinder's heat pump coil heats only the top 1/2 of the cylinder. At times of excess PV, the whole 
    cylinder is heated due to the de-strat circulation.   The reason of ths approach is due to a lack
    of control inputs to switch an ASHP on, or up.
    
  Use CT4 with HP because its more accurate to monitor the standby power of heat pump.
  CT1 mains inp/exp,  CT2 PV, CT3 Immersun (or other), CT4 Heat Pump
  Due to power requirements of relay, this EmonTx must be powered by USB  (not the 9vac sensor)
  If AC-AC adapter is detected, assume emonTx is also powered from adapter (jumper shorted) and take Real Power Readings and disable sleep mode to keep load on power supply constant
  If AC-AC adapter is not detected, assume powering from batteries / USB 5V, AC sample is not present so take Apparent Power Readings and enable sleep mode
  
  Transmit values via RFM69CW radio to Pi
  
   -----------------------------------------
  Part of the openenergymonitor.org project
  
  Authors: Glyn Hudson & Trystan Lea 
  Adaped crudely by John Cantor November 2016
  Builds upon JCW JeeLabs RF12 library and Arduino 
  
  Licence: GNU GPL V3


*/

#define emonTxV3                                               // Tell emonLib this is the emonTx V3 - don't read Vcc assume Vcc = 3.3V as is always the case on emonTx V3 eliminates bandgap error and need for calibration http://harizanov.com/2013/09/thoughts-on-avr-adc-accuracy/
#define RF69_COMPAT 1                                          // Set to 1 if using RFM69CW or 0 is using RFM12B
#include <JeeLib.h>                                            //https://github.com/jcw/jeelib - Tested with JeeLib 3/11/14
ISR(WDT_vect) { Sleepy::watchdogEvent(); }                     // Attached JeeLib sleep function to Atmega328 watchdog -enables MCU to be put into sleep mode inbetween readings to reduce power consumption 

#include "EmonLib.h"                                           // Include EmonLib energy monitoring library https://github.com/openenergymonitor/EmonLib
EnergyMonitor ct1, ct2, ct3, ct4;       

#include <OneWire.h>                                           //http://www.pjrc.com/teensy/td_libs_OneWire.html
#include <DallasTemperature.h>                                 //http://download.milesburton.com/Arduino/MaximTemperature/DallasTemperature_LATEST.zip


const byte version = 18;      //QUERY             // CHECK. firmware version divided by 10 e,g 16 = V1.6

//----------------------------emonTx V3 Settings---------------------------------------------------------------------------------------------------------------
const byte Vrms=                  230;                               // Vrms for apparent power readings (when no AC-AC voltage sample is present)
const byte TIME_BETWEEN_READINGS= 10;            //Time between readings   

//http://openenergymonitor.org/emon/buildingblocks/calibration

const float Ical1=                90.9;                                 // (2000 turns / 22 Ohm burden) = 90.9
const float Ical2=                90.9;                                 // (2000 turns / 22 Ohm burden) = 90.9
const float Ical3=                90.9;                                 // (2000 turns / 22 Ohm burden) = 90.9
const float Ical4=                16.67;                               // (2000 turns / 120 Ohm burden) = 16.67

float Vcal=                       268.97;                             // (230V x 13) / (9V x 1.2) = 276.9 Calibration for UK AC-AC adapter 77DB-06-09 
//float Vcal=276.9;
//const float Vcal=               260;          //QUERY                   //  Calibration for EU AC-AC adapter 77DE-06-09 
const float Vcal_USA=             130.0;                             //Calibration for US AC-AC adapter 77DA-10-09
boolean USA=FALSE; 

const float phase_shift=          1.7;
const int no_of_samples=          1480; 
const int no_of_half_wavelengths= 20;
const int timeout=                2000;                               //emonLib timeout 
const int ACAC_DETECTION_LEVEL=   3000;
const byte min_pulsewidth= 110;                             // minimum width of interrupt pulse (default pulse output meters = 100ms)
const int TEMPERATURE_PRECISION=  12;   // Highest          //9 (93.8ms),10 (187.5ms) ,11 (375ms) or 12 (750ms) bits equal to resplution of 0.5C, 0.25C, 0.125C and 0.0625C
const byte MaxOnewire=             6;                       // EXPECTING 1Flow, 2Return, 3DHW, 4Outside.  possibly 2 more.                     
#define ASYNC_DELAY 750                  //CHANGED          // DS18B20 conversion delay - 9bit requres 95ms, 10bit 187ms, 11bit 375ms and 12bit resolution takes 750ms
//-------------------------------------------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------------------------------------------


//----------------------------emonTx V3 hard-wired connections--------------------------------------------------------------------------------------------------------------- 
const byte LEDpin=                 6;                              // emonTx V3 LED
const byte Relay_Sig=             19;         //CHANGED            // Relay output. WAS DS18b20 PWR.  Note, relay is on when LOW !!!
const byte DIP_switch1=            8;                              // CHANGED.  //WAS Voltage selection 230 / 110 V AC  
const byte DIP_switch2=            9;                              // CHANGED.  //WAS RF node ID 
const byte battery_voltage_pin=    7;                              // Battery Voltage sample from 3 x AA.  NOT NEEDED FOR Solar Relay
const byte pulse_countINT=         1;                              // INT 1 / Dig 3 Terminal Block / RJ45 Pulse counting pin(emonTx V3.4) - (INT0 / Dig2 emonTx V3.2)
const byte pulse_count_pin=        3;                              // INT 1 / Dig 3 Terminal Block / RJ45 Pulse counting pin(emonTx V3.4) - (INT0 / Dig2 emonTx V3.2)
#define ONE_WIRE_BUS               5                               // DS18B20 Data                     
//-------------------------------------------------------------------------------------------------------------------------------------------
//  TESTING ONLY, delete later   Used to check relay operation
int mainsTest = -900;         // export is a negative number
int PVTest = 2000;
int ImmsTest = 23;
int HPtest = 24;

// Added section for PV relay  PLEASE AMMEND START THRESHOLDS DEPENDNG ON HEAT PUMP INPUT POWER.  
//If threshold too low, the heat pump may start when sun variable - it might use imported mains.  If threshold too high, it may miss the available sun
//In general, if PV small, set threshold lower.  If PV big, set higher and wait for period with more PV.
//       The 4 threshold levels options are switchable using dip switches on the EmonTx.V3.4 pcb
int OnThreshold1 = -900;           // (-900) Export needs to be higher (more negative) than this value before 'count'(delay) to start de-strat circulator or heat pump.
int OnThreshold2 = -1000;          // (-1000) Export needs to be higher than this value before 'count'(delay) to start de-strat circulator or heat pump.
int OnThreshold3 = -1100;          // (-1100) Export needs to be higher than this value before 'count'(delay) to start de-strat circulator or heat pump.
int OnThreshold4 = -1200;          // (-1200) Export needs to be higher than this value before 'count'(delay) to start de-strat circulator or heat pump.
int StopThresh = 0;                // (0) watts import threshold for switch-OFF of relay1.  (a negitive number switches off more) A positive number allows the heat pump to import a small amount.  It could be zero
int countmax = 12;                  // (+12)(2 mins) For delay start. Just a number counting up for each (10 second) light-flash. Circ pump needs short delay. Heat pump needs 10 or 15 mins delay
int stopcountmin = -6;             // (-6)(1 min)  Stop delay (negative number) is the number that the stop countd down to before stopping.  Circ pump needs no delay. Heat pump needs 2 or 5 mins delay
int count =0;                      // This counts up if there is net export etc. (delay before relay switching on). It is a safeguards against rapid short-cycling
int stopcount =0;                  // This counts down if net import etc  (delay before turning relay off)
int OnThreshold =0;                // This value is chosed from 1 to 4 below. It depends on Dip switch positions
int Relay1 = 0;                    // use as status flag for heat pump relay status (sent to emoncms)
int Dip1  ;                        // Dip1 & Dip2 switches to set PV threshold for switching on
int Dip2  ;
int DipFactor ;                    // used to select thereshold level
//  End PV relay section
int PVused ;                        // This sends PV used on site as posted to the PV exported   
int PV2HP ;                         // This is HP power from PV.  If PV > HP, PV2HP = HP,  If PV < HP, PV2HP = PV              
int HPpercentPV;                      // WORK NEEDED.  To test if this device is improving things, shows %age HP on PV as opposed to PV from mains
//Setup DS128B20
OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature sensors(&oneWire);
byte allAddress [MaxOnewire][8];  // 8 bytes per address
byte numSensors;
//-------------------------------------------------------------------------------------------------------------------------------------------

//-----------------------RFM12B / RFM69CW SETTINGS----------------------------------------------------------------------------------------------------
#define RF_freq RF12_433MHZ                                              // Frequency of RF69CW module can be RF12_433MHZ, RF12_868MHZ or RF12_915MHZ. You should use the one matching the module you have.
byte nodeID = 9;                                                // emonTx RFM12B node ID
const int networkGroup = 210;  
typedef struct { 
int Pwr1Mains, Pwr2PV, Pwr3Imms, Pwr4HP, Vrms, temp[MaxOnewire], OnThreshold, PVused, PVHP , Relay1, count, stopcount,HPpercentPV ;    // Note, switch positions added as 1,2,3 or 4 to end of threshold number
int pulseCount; 
} PayloadTX;     // create structure - a neat way of packaging data for RF comms
  PayloadTX emontx; 
//-------------------------------------------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------------------------------------------


//Random Variables 
//boolean settled = false;
boolean CT1, CT2, CT3, CT4, ACAC, debug, DS18B20_STATUS; 
byte CT_count=0;
volatile byte pulseCount = 0;
unsigned long pulsetime=0;                                      // Record time of interrupt pulse        

unsigned long lasttime = 0;                                   // ADDED

void setup()
{ 
  pinMode(LEDpin, OUTPUT); 
  pinMode(Relay_Sig, OUTPUT);                    // Changed DS18b20_PWR to Relay_Signal output. 
  digitalWrite(Relay_Sig,HIGH);                  // IMPORTANT; Set to HIGH.   Relay is off when input is HIGH 
  
  pinMode(pulse_count_pin, INPUT);                             // Set emonTx V3.4 interrupt pulse counting pin as input (Dig 3 / INT1)
  emontx.pulseCount=0;                                        // Make sure pulse count starts at zero

  digitalWrite(LEDpin,HIGH); 

  Serial.begin(9600);
 
 Serial.print(" emonTx V3.4 PV_relay_Oct2016c  "); Serial.print(version*0.1); 
  #if (RF69_COMPAT)
    Serial.println(" RFM69CW");
  #else
    Serial.println(" RFM12B");
  #endif
  Serial.println("OpenEnergyMonitor.org");
  Serial.println("POST.....wait 10s");
  
  //READ DIP SWITCH POSITIONS 
  pinMode(DIP_switch1, INPUT_PULLUP);
  pinMode(DIP_switch2, INPUT_PULLUP);
// Moved switch changes from here
 

 if (USA==TRUE){                                                                             //if USA mode is true
  Vcal=Vcal_USA;                                                                               //Assume USA AC/AC adatper is being used, set calibration accordingly 
 } 
  
  delay(10);
  rf12_initialize(nodeID, RF_freq, networkGroup);                          // initialize RFM12B/rfm69CW
   for (int i=10; i>=0; i--)                                                                  //Send RF test sequence (for factory testing)
   {
     emontx.Pwr1Mains=i;
     rf12_sendNow(0, &emontx, sizeof emontx);
     delay(100);
   }
  rf12_sendWait(2);
  emontx.Pwr1Mains=0;
 
 
  

  if (analogRead(1) > 0) {CT1 = 1; CT_count++;} else CT1=0;              // check to see if CT is connected to CT1 input, if so enable that channel
  if (analogRead(2) > 0) {CT2 = 1; CT_count++;} else CT2=0;              // check to see if CT is connected to CT2 input, if so enable that channel
  if (analogRead(3) > 0) {CT3 = 1; CT_count++;} else CT3=0;              // check to see if CT is connected to CT3 input, if so enable that channel
  if (analogRead(4) > 0) {CT4 = 1; CT_count++;} else CT4=0;              //  check to see if CT is connected to CT4 input, if so enable that channel
  
  if ( CT_count == 0) CT1=1;                                              // If no CT's are connect ed CT1-4 then by default read from CT1

  // Quick check to see if there is a voltage waveform present on the ACAC Voltage input
  // Check consists of calculating the RMS from 100 samples of the voltage input.
  Sleepy::loseSomeTime(10000);            //wait for settle
  digitalWrite(LEDpin,LOW); 
  
  // Calculate if there is an ACAC adapter on analog input 0
  //double vrms = calc_rms(0,1780) * (Vcal * (3.3/1024) );
  double vrms = calc_rms(0,1780) * 0.87;
  if (vrms>90) ACAC = 1; else ACAC=0;
 
  if (ACAC) 
  {
    for (int i=0; i<10; i++)                                              // indicate AC has been detected by flashing LED 10 times
    { 
      digitalWrite(LEDpin, HIGH); delay(200);
      digitalWrite(LEDpin, LOW); delay(300);
    }
  }
  else 
  {
    delay(1000);
    digitalWrite(LEDpin, HIGH); delay(2000); digitalWrite(LEDpin, LOW);   // indicate DC power has been detected by turing LED on then off
  }
 
 
 //################################################################################################################################
  //Setup and for presence of DS18B20
  //################################################################################################################################
  //digitalWrite(DS18B20_PWR, HIGH); delay(100);     //REMOVE due to Relay
  sensors.begin();
  sensors.setWaitForConversion(false);             //disable automatic temperature conversion to reduce time spent awake, conversion will be implemented manually in sleeping http://harizanov.com/2013/07/optimizing-ds18b20-code-for-low-power-applications/ 
  numSensors=(sensors.getDeviceCount());
  if (numSensors > MaxOnewire) numSensors=MaxOnewire;   //Limit number of sensors to max number of sensors 
  
  byte j=0;                                        // search for one wire devices and
                                                   // copy to device address arrays.
  while ((j < numSensors) && (oneWire.search(allAddress[j])))  j++;
  
  delay(500);
  //digitalWrite(DS18B20_PWR, LOW);                        //REMOVE due to Relay
  
  if (numSensors==0) DS18B20_STATUS=0; 
    else DS18B20_STATUS=1;

  
  //################################################################################################################################

  if (Serial) debug = 1; else debug=0;          //if serial UART to USB is connected show debug O/P. If not then disable serial
  if (debug==1)
  {
    Serial.print("CT 1 Cal "); Serial.println(Ical1);
    Serial.print("CT 2 Cal "); Serial.println(Ical2);
    Serial.print("CT 3 Cal "); Serial.println(Ical3);
    Serial.print("CT 4 Cal "); Serial.println(Ical4);
    delay(1000);

    Serial.print("RMS Voltage on AC-AC  is: ~");
    Serial.print(vrms,0); Serial.println("V");
      
    if (ACAC) 
    {
      Serial.println("AC-AC detected - Real Power measure enabled");
      Serial.println("assuming pwr from AC-AC (jumper closed)");
      if (USA==TRUE) Serial.println("USA mode active"); 
      Serial.print("Vcal: "); Serial.println(Vcal);
      Serial.print("Phase Shift: "); Serial.println(phase_shift);
    }
     else 
     {
       Serial.println("AC-AC NOT detected - Apparent Pwr measure enabled");
       Serial.print("Assuming VRMS: "); Serial.print(Vrms); Serial.println("V");
       Serial.println("Assuming power from batt / 5V USB - power save enabled");
     }  

    if (CT_count==0) Serial.println("NO CT's detected");
    else   
    {
      if (CT1) Serial.println("CT 1 detected");
      if (CT2) Serial.println("CT 2 detected");
      if (CT3) Serial.println("CT 3 detected");
      if (CT4) Serial.println("CT 4 detected");
    }
    if (DS18B20_STATUS==1) {Serial.print("Detected Temp Sensors:  "); Serial.println(numSensors);}
      else Serial.println("No temperature sensor");
   
    #if (RF69_COMPAT)
       Serial.println("RFM69CW");
    #else
      Serial.println("RFM12B");
    #endif
    
    
    Serial.print("Node: "); Serial.print(nodeID); 
    Serial.print(" Freq: "); 
    if (RF_freq == RF12_433MHZ) Serial.print("433Mhz");
    if (RF_freq == RF12_868MHZ) Serial.print("868Mhz");
    if (RF_freq == RF12_915MHZ) Serial.print("915Mhz"); 
    Serial.print(" Network: "); Serial.println(networkGroup);

    Serial.print("CT1manis CT2PV CT3imms CT4HP VRMS T1 T2 T3 T4");
    if (DS18B20_STATUS==1){Serial.print(" Temperature 1-"); Serial.print(numSensors);}
    Serial.println(" "); 
   delay(500);  

  }
  else 
    Serial.end();
    
  if (CT1) ct1.current(1, Ical1);             // CT ADC channel 1, calibration.  calibration (2000 turns / 22 Ohm burden resistor = 90.909)
  if (CT2) ct2.current(2, Ical2);             // CT ADC channel 2, calibration.
  if (CT3) ct3.current(3, Ical3);             // CT ADC channel 3, calibration. 
  //CT 3 is high accuracy @ low power -  4.5kW Max @ 240V 
  if (CT4) ct4.current(4, Ical4);                                      // CT channel ADC 4, calibration.    calibration (2000 turns / 120 Ohm burden resistor = 16.66)
  
  if (ACAC)
  {
    if (CT1) ct1.voltage(0, Vcal, phase_shift);          // ADC pin, Calibration, phase_shift
    if (CT2) ct2.voltage(0, Vcal, phase_shift);          // ADC pin, Calibration, phase_shift
    if (CT3) ct3.voltage(0, Vcal, phase_shift);          // ADC pin, Calibration, phase_shift
    if (CT4) ct4.voltage(0, Vcal, phase_shift);          // ADC pin, Calibration, phase_shift
  }

attachInterrupt(pulse_countINT, onPulse, FALLING);     // Attach pulse counting interrupt pulse counting 
 
} //end SETUP

void loop()

{
        unsigned long now = millis();
   if   (digitalRead(DIP_switch1)==LOW)      Dip1= 0;   else Dip1 = 2;               //set switching threshold level (see formula DipFactor) ;                    //Originally- IF DIP switch 1 is switched on then subtract 1 from nodeID
   if   (digitalRead(DIP_switch2)==LOW)      Dip2= 1;   else Dip2 = 2;               //set switching threshold level;                    //Originally -IF DIP switch 2 is switched on then activate USA mode

DipFactor = Dip1 + Dip2;      // DipFactor is 1,2,3 or 4 depending on switch positions

Serial.print (DipFactor) ; 
Serial.println("dipfactor");

//====================================================
switch (DipFactor) {
    case 1:
     OnThreshold = (OnThreshold1 - DipFactor);              // add DipFactor 'code' (1,2,3 or 4) to threshold to see what actual switch position is on emoncms
      break;
    case 2:
      OnThreshold = (OnThreshold2 - DipFactor);
      break;
    case 3:
      OnThreshold = (OnThreshold3 - DipFactor);
      break;
    case 4:
      OnThreshold = (OnThreshold4 - DipFactor);
      break;
  }

  Serial.print (OnThreshold) ; 
Serial.println("OnThreshold");
//=====================================================
  
  if (ACAC) {
    delay(200);                         //if powering from AC-AC allow time for power supply to settle    
    emontx.Vrms=0;                      //Set Vrms to zero, this will be overwirtten by CT 1-4
  }
  
  if (CT1) 
  {
   if (ACAC) 
   {
     ct1.calcVI(no_of_half_wavelengths,timeout); emontx.Pwr1Mains=ct1.realPower;
     emontx.Vrms=ct1.Vrms*100;
   }
   else
     emontx.Pwr1Mains = (mainsTest); // CHANGED for TEST! please delete later = ct1.calcIrms(no_of_samples)*Vrms;                               // Calculate Apparent Power 1  1480 is  number of sample

  }
  
  if (CT2) 
  {
   if (ACAC) 
   {
     ct2.calcVI(no_of_half_wavelengths,timeout); emontx.Pwr2PV=ct2.realPower;
     emontx.Vrms=ct2.Vrms*100;
   }
   else
     emontx.Pwr2PV =    (PVTest) ;    //CHANGED for TEST! please delete later             = ct2.calcIrms(no_of_samples)*Vrms;                               // Calculate Apparent Power 1  1480 is  number of samples

  }

  if (CT3) 
  {
   if (ACAC) 
   {
     ct3.calcVI(no_of_half_wavelengths,timeout); emontx.Pwr3Imms=ct3.realPower;
     emontx.Vrms=ct3.Vrms*100;
   }
   else
     emontx.Pwr3Imms =    (ImmsTest);           //CHANGED for TEST! please delete later          = ct3.calcIrms(no_of_samples)*Vrms;                               // Calculate Apparent Power 1  1480 is  number of samples

  }
  

  if (CT4) 
  {
   if (ACAC) 
   {
     ct4.calcVI(no_of_half_wavelengths,timeout); emontx.Pwr4HP=ct4.realPower;
     emontx.Vrms=ct4.Vrms*100;
   }
   else
     emontx.Pwr4HP = (HPtest);                   //CHANGED for TEST! please delete later            = ct4.calcIrms(no_of_samples)*Vrms;                               // Calculate Apparent Power 1  1480 is  number of samples
  }
  
  if (!ACAC){                                                                                         //read battery voltage if powered by DC
    int battery_voltage=analogRead(battery_voltage_pin) * 0.681322727;     //6.6V battery = 3.3V input = 1024 ADC
    emontx.Vrms= battery_voltage;
  }

  
  if (DS18B20_STATUS==1) 
    {
      //digitalWrite(DS18B20_PWR, HIGH); Sleepy::loseSomeTime(50); //REMOVE due to Relay
      for(int j=0;j<numSensors;j++) sensors.setResolution(allAddress[j], TEMPERATURE_PRECISION);      // and set the a to d conversion resolution of each.
      sensors.requestTemperatures();
      Sleepy::loseSomeTime(ASYNC_DELAY); //Must wait for conversion, since we use ASYNC mode 
      for(byte j=0;j<numSensors;j++) emontx.temp[j]=get_temperature(j); 
      //digitalWrite(DS18B20_PWR, LOW);                              //REMOVE due to Relay
    } 
  
  if (pulseCount)                                                       // if the ISR has counted some pulses, update the total count
  {
    cli();                                                             // Disable interrupt just in case pulse comes in while we are updating the count
    emontx.pulseCount += pulseCount;
    pulseCount = 0;
    sei();                                                            // Re-enable interrupts
  }
 emontx.OnThreshold = OnThreshold;                                   //  Send threshold (with switch suffix 1,2,3 or4) 
 emontx.count = count;                                  //  send count for testing
 emontx.stopcount = stopcount;                          //  stopcount for testing
  if (debug==1) {
    Serial.print(emontx.Pwr1Mains); Serial.print(" ");
    Serial.print(emontx.Pwr2PV); Serial.print(" ");
    Serial.print(emontx.Pwr3Imms); Serial.print(" ");
    Serial.print(emontx.Pwr4HP); Serial.print(" ");
    Serial.print(emontx.Vrms); Serial.print(" ");
   // Serial.print(emontx.pulseCount); Serial.print(" ");
    //Serial.print(emontx.Dip1); Serial.print(" dip1 ");
    //Serial.print(emontx.Dip2); Serial.print(" dip2 ");
    Serial.print(emontx.OnThreshold); Serial.print(" On threshold ");
    
    if (DS18B20_STATUS==1){
      for(byte j=0;j<numSensors;j++){
        Serial.print(emontx.temp[j]);
       Serial.print(" ");
      } 
    }
    Serial.println(" ");
    delay(50);
  } 
  //================================== calculate PV used in house
  if ( emontx.Pwr1Mains >= 0 )    //If mains is positive, we are importing.  Any PV will be used directly and none exported
      
     {   emontx.PVused =   emontx.Pwr2PV;
    }
    
    if (emontx.Pwr1Mains <0)   // If mains is negative, we are exporting.   PV used = PV minus export.    Export is negative therefore PV + (-mains) = PV + mains.
    {        emontx.PVused =   emontx.Pwr2PV + emontx.Pwr1Mains  ;
    }
    
    if (emontx.PVused < 0)        // if PV ever recorded below zero, it wont show negatine PV use.  However, this should never happen
    { 
        emontx.PVused = 0  ;        
    }
    Serial.print (" PVused ");
    Serial.println(emontx.PVused);         
    delay(100);
  //================================== END calculate PV used in house 
  //================================== calculate PV used by HP
  if ( emontx.Pwr2PV >= emontx.Pwr4HP )    //If PV greater than HP, HP using all PV. (debatable!!)
      
     {   emontx.PVHP =  emontx.Pwr4HP ;
    }
    if ( emontx.Pwr2PV < emontx.Pwr4HP )    //If PV less than HP.  Heat pump using part PV and part mains.  proprition of PV used = total PV input
      
     {   emontx.PVHP =  emontx.Pwr2PV ;
    }
      Serial.print (" PVHP ");
    Serial.println(emontx.PVHP);         
    delay(100);
  //================================== END calculate PV used by HP
 //================================== Now work out a metric for net worth of HP running WRT PV available
// Dont know best way to do this.  Help needed

  //================================== relay switching
       if ((now-lasttime)>1000) {
        lasttime = now;

 if ((emontx.Pwr1Mains < ( OnThreshold ) && ( emontx.Pwr2PV > (OnThreshold *-1) ) ) )   //need to count up if this is true.  Export to be more than HP requirement, also, PV must be at least this.
                      
        {
            count ++; 
        } else {
            count --;                // if above threshold, count up.  else, count down.
        }
         Serial.print (" count ");
    Serial.println(emontx.count);         
    delay(100);
    
        if (count > countmax)

   { digitalWrite(Relay_Sig, LOW); delay(100), (Relay1 = 1) ; delay (20); (count = 0) ; delay(20); }   //switch on relay LOW = on, relay =1
      
       }
           if (count < 0 ) count =0;                             // Stop count going below zero

//====================Now stop (close) relay

            if (emontx.Pwr1Mains > StopThresh  )  //if import, start stopcount.  If destrat pump, should start stopping early
        {
           stopcount --; 
        } else  {
           stopcount ++;
        }
         Serial.print (" stopcount ");
    Serial.println(emontx.stopcount);         
    delay(100);
    
        if (stopcount < stopcountmin) {
        digitalWrite(Relay_Sig, HIGH); delay(100); Relay1 = 0;  delay (20); stopcount = 0; delay(20);    //switch off relay (high = off)
      }    
          emontx.Relay1 = Relay1;                  // send relay status
        if (stopcount > 0 ) stopcount =0;          // stops stopcount going above zero

       Serial.print (" Relay ");
    Serial.println(emontx.Relay1);         
    delay(100);
  //================================== END relay switching
                                             //  

  if (ACAC) {digitalWrite(LEDpin, HIGH); delay(200); digitalWrite(LEDpin, LOW);}    // flash LED if powered by AC
  
  send_rf_data();                                                       // *SEND RF DATA* - see emontx_lib
    
    if (ACAC)                                                               //If powered by AC-AC adaper (mains power) then delay instead of sleep
    {
     delay(TIME_BETWEEN_READINGS*1000);
    }
    
    else                                                                  //if powered by battery then sleep rather than dealy and disable LED to lower energy consumption  
      Sleepy::loseSomeTime(TIME_BETWEEN_READINGS*1000);                                  // sleep or delay in seconds 

}

void send_rf_data()
{
  rf12_sleep(RF12_WAKEUP);                                   
  rf12_sendNow(0, &emontx, sizeof emontx);                           //send temperature data via RFM12B using new rf12_sendNow wrapper
  rf12_sendWait(2);
  if (!ACAC) rf12_sleep(RF12_SLEEP);                             //if powred by battery then put the RF module into sleep inbetween readings 
}


double calc_rms(int pin, int samples)
{
  unsigned long sum = 0;
  for (int i=0; i<samples; i++) // 178 samples takes about 20ms
  {
    int raw = (analogRead(0)-512);
    sum += (unsigned long)raw * raw;
  }
  double rms = sqrt((double)sum / samples);
  return rms;
}

// The interrupt routine - runs each time a falling edge of a pulse is detected
void onPulse()                  
{  
  if ( (millis() - pulsetime) > min_pulsewidth) {
    pulseCount++;                    //calculate wh elapsed from time between pulses
    pulsetime=millis(); 
  }    
}

int get_temperature(byte sensor)                
{
  float temp=(sensors.getTempC(allAddress[sensor]));
  if ((temp<125.0) && (temp>-55.0)) return(temp*10);            //if reading is within range for the sensor convert float to int ready to send via RF
}
``````````````````````````````````
